using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using Microsoft.Build.BuildEngine;
using Microsoft.Build.Framework;

namespace EmailLogger
{
    /// <summary>
    /// MSBuild logger that will send emails when an error occurs.
    /// <author>Sayed Ibrahim Hashimi (sayed.hashimi@gmail.com) 2/07/2005</author>
    /// </summary>
    public class EmailLogger : FileLogger
    {

        #region Fields

        private IDictionary<string, string> properties;
        
        private bool errorOccurred = false;
        private string logFile;
        #endregion

        public EmailLogger()
            : base()
        {
            this.properties = new Dictionary<string, string>();
            this.logFile = "msbuild.log";   //defult log file name for FileLogger
        }
        private void SetParameters(string paramString)
        {
            //all we care about is the LOGFILE value
            string[] parameters = paramString.Split(";".ToCharArray());
            foreach (string nameValue in parameters)
            {
                string[] both = nameValue.Split("=".ToCharArray());
                if (both == null || both.Length <= 1)
                {
                    continue;
                }
                string paramName = both[0].Trim();
                string paramValue = both[1].Trim();
                if (string.Compare(paramName, "Logfile", true) == 0)
                {
                    this.logFile = paramValue;
                    return; //not looking for any others
                }

            }
        }
        #region Logger methods
        /// <summary>
        /// This is guarenteed to be called by MSBuild before any build steps occur
        /// </summary>
        /// <param name="eventSource"></param>
        public override void Initialize(Microsoft.Build.Framework.IEventSource eventSource)
        {
            //This call will set the correct values from the parameters
            base.Initialize(eventSource);
            this.SetParameters(base.Parameters);
            //add the logger delegates here
            eventSource.ErrorRaised += new BuildErrorEventHandler(this.BuildError);
            eventSource.ProjectStarted += new ProjectStartedEventHandler(this.ProjectStarted);
        }

        /// <summary>
        /// The event that is raised when the project is started.
        /// This will examine all of the properties defined in the project file
        /// and store them in the <code>properties</code> dictionary
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ProjectStarted(object sender, ProjectStartedEventArgs e)
        {
            if (e.Properties != null)
            {
                foreach (DictionaryEntry entry in e.Properties)
                {
                    this.properties.Add(entry.Key.ToString(), entry.Value.ToString());
                }
            }
        }
        /// <summary>
        /// This is called in the event of an error.
        /// In this example we just mark that there was an error and the email is sent
        /// at the end of the build in the <code>Shutdown</code> method.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void BuildError(object sender, BuildErrorEventArgs e)
        {
            this.errorOccurred = true;
        }

        /// <summary>
        /// This is called before msbuild exits.
        /// This is the method that will actually send the email out if there was an error
        /// or if send on success was specified as a property to 'true'
        /// </summary>
        public override void Shutdown()
        {
            //this should close down the log file
            base.Shutdown();

            //get the values for the email from the properties            

            if (this.properties == null || this.properties.Count <= 0)
            {
                return;
            }

            Emailer emailer = new Emailer();

            emailer.SmtpHost = this.GetProperty(EmailLoggerStrings.SmtpHostString);
            emailer.From = this.GetProperty(EmailLoggerStrings.FromString);
            emailer.To = this.GetProperty(EmailLoggerStrings.ToStr);
            emailer.Cc = this.GetProperty(EmailLoggerStrings.CcString);
            emailer.SetSmtpPort(this.GetProperty(EmailLoggerStrings.SmtpPortString));
            emailer.SetTimeout(this.GetProperty(EmailLoggerStrings.TimeoutString));

            bool emailOnSuccess = false;
            string emailOnSuccessStr = this.properties[EmailLoggerStrings.EmailOnSuccessString];
            if (!string.IsNullOrEmpty(emailOnSuccessStr))
            {
                emailOnSuccess = bool.Parse(emailOnSuccessStr);
            }
            if ( errorOccurred || emailOnSuccess )
            {
                //we need to send to email the log file to recipient here
                emailer.Subject = "Build log " + DateTime.Now.ToShortDateString();
                emailer.Body = "Log file attached";
                emailer.Attachment = this.logFile;

                emailer.SendEmail();
            }
        }
        #endregion
        string GetProperty(string name)
        {
            if (this.properties == null || this.properties.Keys.Count <= 0)
            {
                return string.Empty;
            }

            string propValue;
            this.properties.TryGetValue(name, out propValue);

            return propValue;
        }

    }
    /// <summary>
    /// Names of strings that are going to be looked for in the MSBuild properties
    /// </summary>
    public static class EmailLoggerStrings
    {
        public static readonly string SmtpHostString = "EmailLogger_SmtpHost";
        public static readonly string FromString = "EmailLogger_From";
        public static readonly string ToStr = "EmailLogger_To";
        public static readonly string CcString = "EmalLogger_CC";
        public static readonly string SmtpPortString = "EmailLogger_SmtpPort";
        public static readonly string TimeoutString = "EmailLogger_TimeOut";
        public static readonly string EmailOnSuccessString = "EmailLogger_SendOnSuccess";
    }
}
